unit main;

interface

uses
    Windows,
    Messages,
    SysUtils,
    Variants,
    Classes,
    Graphics,
    Controls,
    Forms,
    Dialogs,
    GLScene,
    GLObjects,
    GLMisc,
    GLWin32Viewer,
    GLCadencer,
    keyboard,
    GLVectorFileObjects,
    glfilems3d,
    tga,
    GLTexture,
    GLGeomObjects,
    GLThorFX,
    GLHUDObjects,
    GLBitmapFont,
    GLWindowsFont,
    vectorgeometry,
    GLAnimatedSprite,
    vectorlists,
    OpenGL1x,
    AsyncTimer,
    math,
    MPlayer;

type
    TGLAnimatedHUDSprite = class(TGLSprite)
    private
        { Private Declarations }
        fFramesPerRow, fFrameCount, fFrameIndex, fFPS, fTexSize: Integer;
        currtime: single;
    protected
        { Protected Declarations }

    public
        { Public Declarations }
        constructor Create(AOwner: TComponent); override;
        procedure DoRender(var rci: TRenderContextInfo;
            renderSelf, renderChildren: Boolean); override;

        procedure Update(deltatime: single);
        property FramesPerRow: Integer read fFramesPerRow write fFramesPerRow;
        property FrameCount: integer read fFrameCount write fFrameCount;
        property FPS: integer read fFPS write fFPS;
        property FrameIndex: integer read fFrameIndex write fFrameindex;
        property texsize: integer read fTexSize write fTexSize;

    end;

type
    TGameState = (gsMenu, gsOptions, gs1P, gs2P);
    TTurnState = (tsNone, tsSelect, ts1UP, ts2UP, tsCPU);
    TSequenceState = (ssNone, ssBegin, ssEnd, ssP1Move, ssP2Move, ssForfeit, ssQuit);
    TFaderState = (fsOut, fsIn);
    TForm1 = class(TForm)
        GLScene1: TGLScene;
        GLSceneViewer1: TGLSceneViewer;
        Cam: TGLCamera;
        CamH: TGLDummyCube;
        CamV: TGLDummyCube;
        GLLightSource1: TGLLightSource;
        GLCadencer1: TGLCadencer;
        matlib: TGLMaterialLibrary;
        Board: TGLDummyCube;
        Centre: TGLFreeForm;
        Pieces: TGLDummyCube;
        Black: TGLFreeForm;
        White: TGLFreeForm;
        PieceProxies: TGLDummyCube;
        BoardProxies: TGLDummyCube;
        menu: TGLDummyCube;
        GLWindowsBitmapFont1: TGLWindowsBitmapFont;
        quittext: TGLHUDText;
        p1text: TGLHUDText;
        p2text: TGLHUDText;
        optionstext: TGLHUDText;
        Options: TGLDummyCube;
        backtext: TGLHUDText;
        fader: TGLHUDSprite;
        Select: TGLDummyCube;
        SelWhite: TGLDummyCube;
        SelBlack: TGLDummyCube;
        GLLightSource2: TGLLightSource;
        SelText: TGLHUDText;
        Hud: TGLDummyCube;
        HudP1: TGLDummyCube;
        HudP1Text: TGLHUDText;
        HudP2Text: TGLHUDText;
        HudP2: TGLDummyCube;
        Valid: TGLDummyCube;
        validMaster: TGLFreeForm;
        ValidProxies: TGLDummyCube;
        Result: TGLDummyCube;
        ResultText: TGLHUDText;
        p1Scoretext: TGLHUDText;
        p2scoretext: TGLHUDText;
        Forfeit: TGLDummyCube;
        ForfeitText: TGLHUDText;
        MessageText: TGLHUDText;
        AsyncTimer1: TAsyncTimer;
        AsyncTimer2: TAsyncTimer;
        Quit: TGLDummyCube;
        QuitConfirmText: TGLHUDText;
        mp1: TMediaPlayer;
    mp2: TMediaPlayer;
        procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
            newTime: Double);
        procedure GLSceneViewer1MouseDown(Sender: TObject;
            Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        procedure GLSceneViewer1MouseUp(Sender: TObject; Button: TMouseButton;
            Shift: TShiftState; X, Y: Integer);
        procedure FormCreate(Sender: TObject);
        procedure FormResize(Sender: TObject);
        procedure FormKeyUp(Sender: TObject; var Key: Word;
            Shift: TShiftState);
        procedure FormDestroy(Sender: TObject);
        procedure AsyncTimer1Timer(Sender: TObject);
        procedure AsyncTimer2Timer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    private
        { Private declarations }
        rMouseDown: boolean;
        mdpos: TPoint;
        BoardArray: array[0..7, 0..7] of TGLProxyObject;
        menuarray: array[0..3] of TGLHudText;
        optionsarray: array[0..0] of TGLHudText;
        ValidMoves, flippers: TIntegerList;
        GameState: TGameState;
        TurnState: TTurnState;
        SeqState: TSequenceState;
        FaderState: TFaderState;
        menuindex, optionsindex: integer;
        fordelay, flipdelay, cpudelay: single;
        hudp1sprite, hudp2sprite: TGLAnimatedHUDSprite;
        procedure addpieces(abegin, aend: integer);
        procedure FillValidMoves(obj: TGLProxyObject);
        procedure fillFlippers(ax, az: integer; p: TGLProxyObject);
        procedure DoFinalCount;
        procedure DoCPUMove;
        procedure Play(filename: string);
        procedure Say(filename: string);
    public
        { Public declarations }

    end;

var
    Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.Play(filename: string);
begin
    if mp1.Mode <> mpPlaying then
    begin
        mp1.close;
        mp1.FileName := extractfilepath(paramstr(0)) + '\media\'+filename+'.wav';
        mp1.Open;
        mp1.Play;
    end;
end;

procedure TForm1.Say(filename: string);
begin
    if mp2.Mode <> mpPlaying then
    begin
        mp2.close;
        mp2.FileName := extractfilepath(paramstr(0)) + '\media\'+filename+'.wav';
        mp2.Open;
        mp2.Play;
    end;
end;

procedure TForm1.DoCPUMove;
var
    r, i, x, z, max, choiceindex: integer;
    p: TglProxyObject;
begin
    //check all valid moves and pick the one with the most flippers
    //simple  but hopefully it works out ok :)
    //on second thought, i'm gonna add a little ellement of surprise
    //
    r := randomrange(0, 10);

    if r > 4 then
    begin
        max := 0;
        choiceindex := -1;
        for i := 0 to validmoves.count - 1 do
        begin
            if validmoves[i] > 0 then
                x := validmoves[i] mod 8
            else
                x := 0;
            if validmoves[i] > 7 then
                z := validmoves[i] div 8
            else
                z := 0;
            fillflippers(x, z, TGLProxyObject(hudp2.children[0]));
            if flippers.count > max then
            begin
                max := flippers.count;
                choiceindex := i;
            end;
        end;
    end
    else
    begin
        choiceindex := randomrange(0, validmoves.count - 1);
    end;

    if choiceindex=-1 then
      choiceindex:=0;
        if validmoves[choiceindex] > 0 then
            x := validmoves[choiceindex] mod 8
        else
            x := 0;
        if validmoves[choiceindex] > 7 then
            z := validmoves[choiceindex] div 8
        else
            z := 0;

        p := TglProxyObject.create(glscene1);

        seqState := ssP2Move;
        p.MasterObject := TGLProxyObject(hudp2.children[0]).masterobject;
        p.Position.SetPoint((x - 3), 0, (z - 3));
        pieceproxies.AddChild(p);
        boardarray[x, z] := p;
        flipdelay := 0;

        play('flip');
        fillFlippers(x, z, p);
end;

constructor TGLAnimatedHUDSprite.Create(AOwner: TComponent);
begin
    inherited;
    ObjectStyle := ObjectStyle + [osDirectDraw, osNoVisibilityCulling];
    Width := 16;
    Height := 16;
    fFramesPerRow := 1;
    fFrameCount := 1;
    fFrameIndex := 0;
    fFPS := 15;
    currtime := 0;
    fTexSize := 0;
end;

procedure TGLAnimatedHUDSprite.Update(deltatime: single);
begin
    currtime := currtime + deltatime;
    if currtime > 1 / fFPS then
    begin
        fframeindex := fframeindex + 1;
        if fframeindex > fframecount - 1 then
            fframeindex := 0;
        currtime := 0;
    end;
end;

procedure TGLAnimatedHUDSprite.DoRender(var rci: TRenderContextInfo;
    renderSelf, renderChildren: Boolean);
var
    vx, vy, vx1, vy1, f: Single;
    tempx, tempy: single;
begin
    if rci.ignoreMaterials then Exit;
    Material.Apply(rci);
    repeat
        if AlphaChannel <> 1 then
            rci.GLStates.SetGLMaterialAlphaChannel(GL_FRONT, AlphaChannel);
        // Prepare matrices
        glMatrixMode(GL_MODELVIEW);
        glPushMatrix;
        glLoadMatrixf(@Scene.CurrentBuffer.BaseProjectionMatrix);
        if rci.renderDPI = 96 then
            f := 1
        else
            f := rci.renderDPI / 96;
        glScalef(2 / rci.viewPortSize.cx, 2 / rci.viewPortSize.cy, 1);
        glTranslatef(f * Position.X - rci.viewPortSize.cx * 0.5,
            rci.viewPortSize.cy * 0.5 - f * Position.Y, Position.Z);
        if Rotation <> 0 then
            glRotatef(Rotation, 0, 0, 1);
        glMatrixMode(GL_PROJECTION);
        glPushMatrix;
        glLoadIdentity;
        glPushAttrib(GL_ENABLE_BIT);
        glDisable(GL_DEPTH_TEST);
        glDepthMask(False);
        // precalc coordinates
        vx := -Width * 0.5 * f; vx1 := vx + Width * f;
        vy := +Height * 0.5 * f; vy1 := vy - Height * f;
        // issue quad
        if frameindex > 0 then
            tempx := (1 / fFramesPerRow) * (frameindex mod fFramesPerRow)
        else
            tempx := 0;

        if frameindex > fFramesPerRow then
            tempy := 1 / (frameindex div fFramesPerRow)
        else
            tempy := 0;

        glBegin(GL_QUADS);
        glNormal3fv(@YVector);
        glTexCoord2f(tempx, tempy); glVertex2f(vx, vy1);
        glTexCoord2f(tempx + (1 / framesperrow), tempy); glVertex2f(vx1, vy1);
        glTexCoord2f(tempx + (1 / framesperrow), tempy + (1 / framesperrow)); glVertex2f(vx1, vy);
        glTexCoord2f(tempx, tempy + (1 / framesperrow)); glVertex2f(vx, vy);
        glEnd;
        // restore state
        glDepthMask(True);
        glPopAttrib;
        glPopMatrix;
        glMatrixMode(GL_MODELVIEW);
        glPopMatrix;
    until not Material.UnApply(rci);
    if Count > 0 then
        Self.RenderChildren(0, Count - 1, rci);
end;

////////////////////////////////////////////////////////////////////////////////

procedure Tform1.DoFinalCount;
var
    x, z: integer;
begin
    hudp1.Tag := 0;
    hudp2.Tag := 0;
    for z := 0 to 7 do
        for x := 0 to 7 do
            if assigned(boardarray[x, z]) then
                if boardarray[x, z].masterobject = TGLProxyObject(hudp1.children[0]).masterobject then
                    hudp1.Tag := hudp1.tag + 1
                else
                    hudp2.Tag := hudp2.tag + 1;
end;

procedure Tform1.fillFlippers(ax, az: integer; p: TGLProxyObject);

    procedure dopz(x, z: integer);
    var
        isflipper: boolean;
        tempz: integer;
    begin
        z := z + 1;
        while (z < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempz := z + 1;
                    isflipper := false;
                    while (tempz < 8) do
                    begin
                        if not assigned(boardarray[x, tempz]) then
                            tempz := 8
                        else
                            if boardarray[x, tempz].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempz := 8;
                        end
                        else
                            tempz := tempz + 1;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        z := z + 1;
                    end
                    else
                        z := 8;
                end
                else
                    z := 8;
            end
            else
                z := 8;
        end;
    end;

    procedure donz(x, z: integer);
    var
        isflipper: boolean;
        tempz: integer;
    begin
        z := z - 1;
        while (z > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempz := z-1;
                    isflipper := false;
                    while (tempz > -1) do
                    begin
                        if not assigned(boardarray[x, tempz]) then
                            tempz := -1
                        else
                            if boardarray[x, tempz].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempz := -1;
                        end
                        else
                            tempz := tempz - 1;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        z := z - 1;
                    end
                    else
                        z := -1;
                end
                else
                    z := -1;
            end
            else
                z := -1;
        end;
    end;

    procedure dopx(x, z: integer);
    var
        isflipper: boolean;
        tempx: integer;
    begin
        x := x + 1;
        while (x < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempx := x + 1;
                    isflipper := false;
                    while (tempx < 8) do
                    begin
                        if not assigned(boardarray[tempx, z]) then
                            tempx := 8
                        else
                            if boardarray[tempx, z].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempx := 8;
                        end
                        else
                            tempx := tempx + 1;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        x := x + 1;
                    end
                    else
                        x := 8;
                end
                else
                    x := 8;
            end
            else
                x := 8;
        end;
    end;

    procedure donx(x, z: integer);
    var
        isflipper: boolean;
        tempx: integer;
    begin
        x := x - 1;
        while (x > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempx := x - 1;
                    isflipper := false;
                    while (tempx > -1) do
                    begin
                        if not assigned(boardarray[tempx, z]) then
                            tempx := -1
                        else
                            if boardarray[tempx, z].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempx := -1;
                        end
                        else
                            tempx := tempx - 1;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        x := x - 1;
                    end
                    else
                        x := -1;
                end
                else
                    x := -1;
            end
            else
                x := -1;
        end;
    end;

    procedure dopxpz(x, z: integer);
    var
        isflipper: boolean;
        tempx, tempz: integer;
    begin
        x := x + 1;
        z := z + 1;
        while (x < 8) and (z < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempz := z + 1;
                    tempx := x + 1;
                    isflipper := false;
                    while (tempx < 8) and (tempz < 8) do
                    begin
                        if not assigned(boardarray[tempx, tempz]) then
                            tempx := 8
                        else
                            if boardarray[tempx, tempz].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempx := 8;
                        end
                        else
                        begin
                            tempx := tempx + 1;
                            tempz := tempz + 1;
                        end;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        x := x + 1;
                        z := z + 1;
                    end
                    else
                        x := 8;
                end
                else
                    x := 8;
            end
            else
                x := 8;
        end;
    end;

    procedure dopxnz(x, z: integer);
    var
        isflipper: boolean;
        tempx, tempz: integer;
    begin
        x := x + 1;
        z := z - 1;
        while (x < 8) and (z > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempz := z - 1;
                    tempx := x + 1;
                    isflipper := false;
                    while (tempx < 8) and (tempz > -1) do
                    begin
                        if not assigned(boardarray[tempx, tempz]) then
                            tempx := 8
                        else
                            if boardarray[tempx, tempz].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempx := 8;
                        end
                        else
                        begin
                            tempx := tempx + 1;
                            tempz := tempz - 1;
                        end;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        x := x + 1;
                        z := z - 1;
                    end
                    else
                        x := 8;
                end
                else
                    x := 8;
            end
            else
                x := 8;
        end;
    end;

    procedure donxnz(x, z: integer);
    var
        isflipper: boolean;
        tempx, tempz: integer;
    begin
        x := x - 1;
        z := z - 1;
        while (x > -1) and (z > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempz := z - 1;
                    tempx := x - 1;
                    isflipper := false;
                    while (tempx > -1) and (tempz > -1) do
                    begin
                        if not assigned(boardarray[tempx, tempz]) then
                            tempx := -1
                        else
                            if boardarray[tempx, tempz].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempx := -1;
                        end
                        else
                        begin
                            tempx := tempx - 1;
                            tempz := tempz - 1;
                        end;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        x := x - 1;
                        z := z - 1;
                    end
                    else
                        x := -1;
                end
                else
                    x := -1;
            end
            else
                x := -1;
        end;
    end;

    procedure donxpz(x, z: integer);
    var
        isflipper: boolean;
        tempx, tempz: integer;
    begin
        x := x - 1;
        z := z + 1;
        while (x > -1) and (z < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> p.MasterObject then
                begin
                    tempz := z + 1;
                    tempx := x - 1;
                    isflipper := false;
                    while (tempx > -1) and (tempz < 8) do
                    begin
                        if not assigned(boardarray[tempx, tempz]) then
                            tempx := -1
                        else
                            if boardarray[tempx, tempz].MasterObject = p.MasterObject then
                        begin
                            isflipper := true;
                            tempx := -1;
                        end
                        else
                        begin
                            tempx := tempx - 1;
                            tempz := tempz + 1;
                        end;
                    end;
                    if isflipper then
                    begin
                        flippers.add((z * 8) + x);
                        x := x - 1;
                        z := z + 1;
                    end
                    else
                        x := -1;
                end
                else
                    x := -1;
            end
            else
                x := -1;
        end;
    end;
begin
    flippers.clear;
    dopz(ax, az);
    donz(ax, az);
    dopx(ax, az);
    donx(ax, az);
    dopxpz(ax, az);
    dopxnz(ax, az);
    donxnz(ax, az);
    donxpz(ax, az);
end;

procedure TForm1.FillValidMoves(obj: TGLProxyObject);
var
    x, z, i: integer;
    curr: TGLProxyObject;
    p: TglProxyObject;

    procedure dopz(x, z: integer);
    begin
        z := z + 1;
        while (z < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    z := z + 1;
                    while (z < 8) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                z := 8;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            z := 8;
                        end;
                        z := z + 1;
                    end;
                end
                else
                    z := 8;
            end
            else
                z := 8;
        end;
    end;

    procedure donz(x, z: integer);
    begin
        z := z - 1;
        while (z > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    z := z - 1;
                    while (z > -1) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                z := -1;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            z := -1;
                        end;
                        z := z - 1;
                    end;
                end
                else
                    z := -1;
            end
            else
                z := -1;
        end;
    end;

    procedure dopx(x, z: integer);
    begin
        x := x + 1;
        while (x < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    x := x + 1;
                    while (x < 8) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                x := 8;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            x := 8;
                        end;
                        x := x + 1;
                    end;
                end
                else
                    x := 8;
            end
            else
                x := 8;
        end;
    end;

    procedure donx(x, z: integer);
    begin
        x := x - 1;
        while (x > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    x := x - 1;
                    while (x > -1) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                x := -1;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            x := -1;
                        end;
                        x := x - 1;
                    end;
                end
                else
                    x := -1;
            end
            else
                x := -1;
        end;
    end;

    procedure dopxpz(x, z: integer);
    begin
        x := x + 1;
        z := z + 1;
        while (x < 8) and (z < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    x := x + 1;
                    z := z + 1;
                    while (x < 8) and (z < 8) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                x := 8;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            x := 8;
                        end;
                        x := x + 1;
                        z := z + 1;
                    end;
                end
                else
                    x := 8;
            end
            else
                x := 8;
        end;
    end;

    procedure dopxnz(x, z: integer);
    begin
        x := x + 1;
        z := z - 1;
        while (x < 8) and (z > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    x := x + 1;
                    z := z - 1;
                    while (x < 8) and (z > -1) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                x := 8;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            x := 8;
                        end;
                        x := x + 1;
                        z := z - 1;
                    end;
                end
                else
                    x := 8;
            end
            else
                x := 8;
        end;
    end;

    procedure donxnz(x, z: integer);
    begin
        x := x - 1;
        z := z - 1;
        while (x > -1) and (z > -1) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    x := x - 1;
                    z := z - 1;
                    while (x > -1) and (z > -1) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                x := -1;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            x := -1;
                        end;
                        x := x - 1;
                        z := z - 1;
                    end;
                end
                else
                    x := -1;
            end
            else
                x := -1;
        end;
    end;

    procedure donxpz(x, z: integer);
    begin
        x := x - 1;
        z := z + 1;
        while (x > -1) and (z < 8) do
        begin
            if assigned(boardarray[x, z]) then
            begin
                if boardarray[x, z].MasterObject <> curr.MasterObject then
                begin
                    x := x - 1;
                    z := z + 1;
                    while (x > -1) and (z < 8) do
                    begin
                        if assigned(boardarray[x, z]) then
                        begin
                            if boardarray[x, z].MasterObject = curr.MasterObject then
                                x := -1;
                        end
                        else
                        begin
                            validmoves.add((z * 8) + x);
                            x := -1;
                        end;
                        x := x - 1;
                        z := z + 1;
                    end;
                end
                else
                    x := -1;
            end
            else
                x := -1;
        end;
    end;
begin
    validmoves.clear;
    validproxies.DeleteChildren;
    for z := 0 to 7 do
        for x := 0 to 7 do
        begin
            curr := boardarray[x, z];
            if assigned(curr) then
            begin
                if curr.MasterObject = obj.MasterObject then
                begin
                    dopz(x, z);
                    donz(x, z);
                    dopx(x, z);
                    donx(x, z);
                    dopxpz(x, z);
                    dopxnz(x, z);
                    donxnz(x, z);
                    donxpz(x, z);
                end;
            end;
        end;
    for i := 0 to validmoves.count - 1 do
    begin
        if validmoves[i] > 0 then
            x := validmoves[i] mod 8
        else
            x := 0;
        if validmoves[i] > 7 then
            z := validmoves[i] div 8
        else
            z := 0;

        p := TglProxyObject.create(glscene1);
        p.MasterObject := validmaster;
        p.Position.SetPoint((x - 3), 0, (z - 3));
        validproxies.AddChild(p);
    end;
end;

procedure TForm1.addpieces(abegin, aend: integer);
var
    p: TGLProxyObject;
    x, z: integer;
begin
    for z := 0 to 7 do
        for x := 0 to 7 do
            boardarray[x, z] := nil;
    pieceproxies.DeleteChildren;
    for z := abegin to aend do
        for x := abegin to aend do
        begin
            p := TglProxyObject.create(glscene1);
            if x mod 2 = 0 then
            begin
                if z mod 2 = 0 then
                    p.MasterObject := black
                else
                    p.MasterObject := white;
            end
            else
                if z mod 2 = 0 then
                p.MasterObject := white
            else
                p.MasterObject := black;
            p.Position.SetPoint(x - 3, 0, z - 3);
            pieceproxies.AddChild(p);
            boardarray[x, z] := p;
        end;
end;

procedure TForm1.GLCadencer1Progress(Sender: TObject; const deltaTime,
    newTime: Double);
var
    pt: TPoint;
    deltax, deltay: single;
    x, z, i: integer;
    bm: TBitmap;

    procedure DoMouse;
    begin
        if rmousedown then
        begin
            GetCursorPos(pt);
            deltax := (mdpos.x - pt.x) / 5;
            deltay := (mdpos.Y - pt.y) / 5;

            CamH.turnangle := CamH.turnangle + deltax;
            if CamH.turnangle >= 360 then
                CamH.turnangle := CamH.turnangle - 360;
            if CamH.turnangle < 0 then
                CamH.turnangle := CamH.turnangle + 360;
            if (CamV.pitchangle + deltay < 40) and (CamV.pitchangle + deltay > -10) then
                CamV.pitchangle := CamV.pitchangle + deltay;

            if (pt.x <> mdpos.x) or (pt.y <> mdpos.y) then
                SetCursorPos(mdpos.x, mdpos.y);
        end;
    end;
begin
    case gamestate of
        gsmenu, gsoptions:
            begin
                camh.turn(deltatime * 20);
                if camh.TurnAngle > 360 then
                    camh.TurnAngle := camh.TurnAngle - 360;
            end;
        gs1P, gs2P:
            begin
                case SeqState of
                    ssBegin:
                        begin
                            case faderstate of
                                fsout:
                                    begin
                                        camh.turn(deltatime * 20);
                                        if camh.TurnAngle > 360 then
                                            camh.TurnAngle := camh.TurnAngle - 360;
                                        fader.Width := fader.width + (deltatime * 1000);
                                        fader.height := fader.height + (deltatime * 1000);

                                        if fader.width > width * 1.5 then
                                        begin
                                            faderstate := fsin;
                                            camh.ResetAndPitchTurnRoll(0, 0, 0);
                                            addpieces(3, 4);
                                            // addpieces(0,0);
                                            //addpieces(7,7);

                                            case turnstate of
                                                tsSelect:
                                                begin
                                                    Select.visible := true;
                                                    say('selectcolor');
                                                end;
                                            end;
                                        end;
                                    end;
                                fsin:
                                    begin
                                        fader.Width := fader.width - (deltatime * 1000);
                                        fader.height := fader.height - (deltatime * 1000);
                                        if fader.width < 1 then
                                        begin
                                            fader.Visible := false;
                                            seqstate := ssNone;
                                        end;
                                    end;
                            end;
                        end;
                    ssEnd:
                        begin
                            camh.turn(deltatime * 20);
                            if camh.TurnAngle > 360 then
                                camh.TurnAngle := camh.TurnAngle - 360;
                            case turnstate of
                                ts1UP:
                                    hudp1sprite.Update(deltatime);
                                ts2UP,tsCPU:
                                    hudp2sprite.Update(deltatime);
                            end;
                        end;
                    ssP1Move, ssP2Move:
                        begin
                            //  for i := 0 to flippers.count - 1 do
                            case seqstate of
                                ssP1Move:
                                    hudp1sprite.Update(deltatime);
                                ssP2Move:
                                    hudp2sprite.Update(deltatime);
                            end;
                            flipdelay := flipdelay + deltatime;
                            if flipdelay > 0.4 then
                            begin
                                flipdelay := 0;
                                if flippers.count > 0 then
                                begin

                                    if flippers[0] > 0 then
                                        x := flippers[0] mod 8
                                    else
                                        x := 0;
                                    if flippers[0] > 7 then
                                        z := flippers[0] div 8
                                    else
                                        z := 0;
                                    if seqstate = ssP2Move then
                                        boardarray[x, z].MasterObject := TGLProxyObject(hudp2.children[0]).masterobject
                                    else
                                        boardarray[x, z].MasterObject := TGLProxyObject(hudp1.children[0]).masterobject;
                                    flippers.Delete(0);
                                    play('flip');
                                    exit;
                                end
                                else
                                begin
                                    if seqstate = ssP2Move then
                                        fillvalidmoves(TGLProxyObject(hudp1.children[0]))
                                    else
                                        fillvalidmoves(TGLProxyObject(hudp2.children[0]));
                                    if validmoves.count = 0 then
                                    begin
                                        if seqstate = ssP2Move then
                                            fillvalidmoves(TGLProxyObject(hudp2.children[0]))
                                        else
                                            fillvalidmoves(TGLProxyObject(hudp1.children[0]));
                                        if validmoves.count = 0 then
                                        begin
                                            seqstate := ssEnd;
                                            DoFinalCount;
                                            if hudp1.Tag > hudp2.Tag then
                                            begin
                                                turnstate:=ts1UP;
                                                resulttext.text := 'Player 1 wins the game.';
                                                say('player1wins');
                                            end
                                            else
                                                if hudp2.Tag > hudp1.Tag then
                                                if gamestate = gs2P then
                                                begin
                                                    turnstate:=ts2UP;
                                                    resulttext.text := 'Player 2 wins the game.';
                                                    say('player2wins');
                                                end
                                                else
                                                begin
                                                    turnstate:=tsCPU;
                                                    resulttext.text := 'CPU wins the game.';
                                                    say('cpuwins');
                                                end
                                            else
                                            begin
                                                resulttext.text := 'The game is tied.';
                                                say('gametied');
                                            end;
                                            p1scoretext.text := inttostr(hudp1.Tag);
                                            p2scoretext.text := inttostr(hudp2.Tag);
                                            result.Visible := true;
                                        end
                                        else
                                        begin
                                            if seqstate = ssP2Move then
                                                Turnstate := ts1UP
                                            else
                                                if gamestate = gs2P then
                                                    Turnstate := ts2UP
                                                else
                                                    Turnstate := tsCPU;
                                            forfeit.Visible := true;
                                            fordelay := 0;
                                            seqstate := ssForfeit;
                                            say('nomoves');
                                        end;
                                    end
                                    else
                                    begin
                                        if seqstate = ssP2Move then
                                            Turnstate := ts1UP
                                        else
                                            if gamestate = gs2P then
                                            Turnstate := ts2UP
                                        else
                                        begin
                                            Turnstate := tsCPU;
                                            cpudelay := 0;
                                        end;
                                        seqstate := ssNone;
                                    end;
                                    hudp2sprite.frameindex := 0;
                                    hudp1sprite.frameindex := 0;
                                end;
                            end;
                        end;
                    ssForfeit:
                        begin
                            case turnstate of
                                ts1UP:
                                    hudp1sprite.Update(deltatime);
                                tsCPU, ts2UP:
                                    hudp2sprite.Update(deltatime);
                            end;
                            fordelay := fordelay + deltatime;
                            if fordelay > 5 then
                            begin
                                fordelay := 0;
                                forfeit.Visible := false;
                                seqstate := ssnone;

                                case turnstate of
                                    ts1UP:
                                        begin
                                            if gamestate = gs1P then
                                                turnstate := tsCPU
                                            else
                                                turnstate := ts2UP;
                                            FillValidMoves(TGLProxyObject(hudp2.Children[0]));
                                        end;
                                    tsCPU, ts2UP:
                                        begin
                                            turnstate := ts1UP;
                                            FillValidMoves(TGLProxyObject(hudp1.Children[0]));
                                        end;
                                end;
                                hudp2sprite.frameindex := 0;
                                hudp1sprite.frameindex := 0;
                            end;
                        end;
                    ssNone:
                        begin
                            valid.Visible := false;
                            if iskeydown(VK_SPACE) then
                            begin
                                valid.Visible := true;
                                cpudelay := 0;
                            end;
                            case turnstate of
                                tsSelect:
                                    begin
                                        selblack.roll(deltatime * 500);
                                        selwhite.roll(-deltatime * 500);
                                    end;
                                ts1UP:
                                    begin
                                        DoMouse;
                                        hudp1sprite.Update(deltatime);
                                    end;
                                ts2Up:
                                    begin
                                        DoMouse;
                                        hudp2sprite.Update(deltatime);
                                    end;
                                tsCPU:
                                    begin
                                        hudp2sprite.Update(deltatime);
                                        cpudelay := cpudelay + deltatime;
                                        if cpudelay > 1 then
                                        begin
                                            DoCPUMove;
                                        end;
                                    end;
                            end;
                        end;
                end;
            end;
    end;
end;

procedure TForm1.GLSceneViewer1MouseDown(Sender: TObject;
    Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    pick: TGLCustomSceneObject;
    i, j, ax, az: integer;
    p: TGLProxyObject;
    isvalid: boolean;
    mat: TGLLibMaterial;

    procedure DoMouse;
    begin
        if button = mbright then
        begin
            rmousedown := true;
            screen.cursor := crNone;
        end;
        GetCursorPos(mdpos);
    end;
begin
    case gamestate of
        gs1P, gs2P:
            begin
                case turnstate of
                    tsSelect:
                        begin
                            if button = mbleft then
                            begin
                                pick := TGLCustomSceneObject(GLSceneViewer1.Buffer.GetPickedObject(x, y));
                                if Assigned(pick) then
                                begin
                                    if (pick = selblack.Children[0]) then
                                    begin
                                        TGLProxyObject(hudp1.Children[0]).masterobject := black;
                                        TGLProxyObject(hudp2.Children[0]).masterobject := white;
                                        turnstate := ts1UP;
                                        hudp1sprite.FrameIndex := 0;
                                        hudp1sprite.Material.Texture.Image.LoadFromFile(extractfilepath(paramstr(0)) + '\media\blackhud.bmp');
                                        hudp2sprite.FrameIndex := 0;
                                        hudp2sprite.Material.Texture.Image.LoadFromFile(extractfilepath(paramstr(0)) + '\media\whitehud.bmp');
                                        select.Visible := false;
                                        hud.Visible := true;
                                        FillValidMoves(TGLProxyObject(hudp1.Children[0]));
                                        play('select');
                                        say('player1starts');
                                    end
                                    else
                                        if (pick = selwhite.children[0]) then
                                    begin
                                        TGLProxyObject(hudp1.Children[0]).masterobject := white;
                                        TGLProxyObject(hudp2.Children[0]).masterobject := black;
                                        if gamestate = gs2P then
                                            turnstate := ts2UP
                                        else
                                            turnstate := tsCPU;
                                        hudp2sprite.FrameIndex := 0;
                                        hudp2sprite.Material.Texture.Image.LoadFromFile(extractfilepath(paramstr(0)) + '\media\blackhud.bmp');
                                        hudp1sprite.FrameIndex := 0;
                                        hudp1sprite.Material.Texture.Image.LoadFromFile(extractfilepath(paramstr(0)) + '\media\whitehud.bmp');
                                        select.Visible := false;
                                        hud.Visible := true;
                                        FillValidMoves(TGLProxyObject(hudp2.Children[0]));
                                        play('select');

                                        if gamestate = gs2P then
                                            say('player2starts')
                                        else
                                            say('cpustarts');

                                    end;

                                end;
                            end;
                        end;
                    ts1UP, ts2UP:
                        begin
                            domouse;
                            if button = mbleft then
                            begin
                                valid.Visible := false;
                                pieces.Visible := false;
                                pick := TGLCustomSceneObject(GLSceneViewer1.Buffer.GetPickedObject(x, y));
                                if Assigned(pick) then
                                begin
                                    if TGLProxyObject(pick) is TGLProxyObject then
                                    begin
                                        if TGLProxyObject(pick).MasterObject = centre then
                                        begin
                                            i := TGLProxyObject(pick).Tag;
                                            isvalid := false;
                                            for j := 0 to validmoves.count - 1 do
                                                if i = validmoves[j] then
                                                begin
                                                    isvalid := true;
                                                    break;
                                                end;

                                            if isvalid then
                                            begin

                                                if i > 0 then
                                                    ax := i mod 8
                                                else
                                                    ax := 0;
                                                if i > 7 then
                                                    az := i div 8
                                                else
                                                    az := 0;

                                                p := TglProxyObject.create(glscene1);
                                                if turnstate = ts1UP then
                                                begin
                                                    seqState := ssP1Move;
                                                    p.MasterObject := TGLProxyObject(hudp1.children[0]).masterobject;
                                                end
                                                else
                                                begin
                                                    seqState := ssP2Move;
                                                    p.MasterObject := TGLProxyObject(hudp2.children[0]).masterobject;
                                                end;
                                                p.Position.SetPoint((ax - 3), 0, (az - 3));
                                                pieceproxies.AddChild(p);
                                                boardarray[ax, az] := p;
                                                flipdelay := 0;

                                                fillFlippers(ax, az, p);
                                                if flippers.count>10 then
                                                  say('excellent')
                                                else
                                                  if flippers.count>6 then
                                                    say('nicemove');
                                                play('flip');
                                            end
                                            else
                                            begin
                                                play('invalid');
                                            end;
                                        end;

                                    end;

                                end;
                                pieces.Visible := true;
                            end;

                        end;

                end;
            end;
    end;
end;

procedure TForm1.GLSceneViewer1MouseUp(Sender: TObject;
    Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if button = mbRight then
    begin
        rmouseDown := false;
        screen.cursor := crDefault;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
    p: TGLProxyObject;
    x, z: integer;
begin
    randomize;
    centre.LoadFromFile(extractfilepath(paramstr(0)) + '\media\board.ms3d');
    matlib.Materials[0].Material.texture.image.loadfromfile(extractfilepath(paramstr(0)) + '\media\felt.tga');
    matlib.Materials[0].Material.texture.disabled := false;
    matlib.Materials[0].Material.texture.texturemode := tmModulate;
    centre.Visible := false;
    for z := 0 to 7 do
        for x := 0 to 7 do
        begin
            //  if (x=3) and (z=3) then
            //    continue;
            p := TglProxyObject.create(glscene1);
            p.Tag := (z * 8) + x;
            p.MasterObject := centre;
            p.Position.SetPoint(x - 3, 0, z - 3);
            boardproxies.AddChild(p);
        end;

    black.LoadFromFile(extractfilepath(paramstr(0)) + '\media\black.ms3d');
    matlib.Materials[1].Material.texture.image.loadfromfile(extractfilepath(paramstr(0)) + '\media\black.tga');
    matlib.Materials[1].Material.texture.disabled := false;
    matlib.Materials[1].Material.texture.texturemode := tmModulate;
    matlib.Materials[1].Material.texture.mappingmode := tmmSphere;
    black.Visible := false;
    white.LoadFromFile(extractfilepath(paramstr(0)) + '\media\white.ms3d');
    matlib.Materials[2].Material.texture.image.loadfromfile(extractfilepath(paramstr(0)) + '\media\white.tga');
    matlib.Materials[2].Material.texture.disabled := false;
    matlib.Materials[2].Material.texture.texturemode := tmModulate;
    matlib.Materials[2].Material.texture.mappingmode := tmmSphere;
    white.Visible := false;

    validmaster.LoadFromFile(extractfilepath(paramstr(0)) + '\media\valid.ms3d');
    matlib.Materials[3].Material.texture.image.loadfromfile(extractfilepath(paramstr(0)) + '\media\valid.tga');
    matlib.Materials[3].Material.texture.disabled := false;
    matlib.Materials[3].Material.blendingmode := bmTransparency;
    matlib.Materials[3].Material.frontproperties.diffuse.alpha := 0.3;

    matlib.Materials[3].Material.texture.mappingmode := tmmSphere;
    validmaster.Visible := false;

    addpieces(0, 7);

    hudp1sprite := TGLAnimatedHUDSprite.create(glscene1);
    hudp1sprite.FramesPerRow := 4;
    hudp1sprite.FrameCount := 8;
    hudp1sprite.FPS := 8;
    hudp1sprite.texsize := 256;
    hudp1sprite.Width := 64;
    hudp1sprite.height := 64;
    hudp1sprite.Material.Texture.Image.LoadFromFile(extractfilepath(paramstr(0)) + '\media\blackhud.bmp');
    hudp1sprite.Material.Texture.TextureMode := tmreplace;
    hudp1sprite.Material.Texture.disabled := false;
    hudp1sprite.Material.MaterialOptions := [moNoLighting];
    hudp1sprite.Material.blendingmode := bmTransparency;
    hudp1sprite.Material.Texture.ImageAlpha := tiaTopLeftPointColorTransparent;

    hudp1sprite.Position.SetPoint(160, 35, 0);
    hud.AddChild(hudp1sprite);

    hudp2sprite := TGLAnimatedHUDSprite.create(glscene1);
    hudp2sprite.FramesPerRow := 4;
    hudp2sprite.FrameCount := 8;
    hudp2sprite.FPS := 8;
    hudp2sprite.texsize := 256;
    hudp2sprite.Width := 64;
    hudp2sprite.height := 64;
    hudp2sprite.Material.Texture.Image.LoadFromFile(extractfilepath(paramstr(0)) + '\media\whitehud.bmp');
    hudp2sprite.Material.Texture.TextureMode := tmreplace;
    hudp2sprite.Material.Texture.disabled := false;
    hudp2sprite.Material.MaterialOptions := [moNoLighting];
    hudp2sprite.Material.blendingmode := bmTransparency;
    hudp2sprite.Material.Texture.ImageAlpha := tiaTopLeftPointColorTransparent;

    hudp2sprite.Position.SetPoint(640, 35, 0);
    hud.AddChild(hudp2sprite);

    p := TglProxyObject.create(glscene1);
    p.MasterObject := black;
    hudp1.Visible := false;
    HudP1.AddChild(p);

    p := TglProxyObject.create(glscene1);
    p.MasterObject := white;
    hudp2.Visible := false;
    Hudp2.AddChild(p);

    p := TglProxyObject.create(glscene1);
    p.MasterObject := black;
    selblack.AddChild(p);

    p := TglProxyObject.create(glscene1);
    p.MasterObject := white;
    selwhite.AddChild(p);

    optionsarray[0] := BackText;

    menuarray[0] := p1text;
    menuarray[1] := p2text;
    menuarray[2] := optionstext;
    menuarray[3] := quittext;
    menuindex := 0;
    GameState := gsMenu;

    ValidMoves := TIntegerlist.Create;
    flippers := TIntegerlist.Create;
end;

procedure TForm1.FormResize(Sender: TObject);
var
    x, y: single;
begin
    //  glsceneviewer1.FieldOfView:=(width/height)*100;
    x := width / 2;
    y := height / 2;
    p1text.Position.SetPoint(x, y - 100, 0);
    p2text.Position.SetPoint(x, y - 30, 0);
    optionstext.Position.SetPoint(x, y + 35, 0);
    quittext.Position.SetPoint(x, y + 100, 0);

    backtext.Position.SetPoint(x, y + 100, 0);

    seltext.Position.SetPoint(x, y - 100, 0);

    HudP2Text.Position.SetPoint(width - 30, 20, 0);

    fader.Position.setpoint(x, y, 0);
    resulttext.Position.SetPoint(x, y - 100, 0);
    p1scoretext.Position.SetPoint(x - 100, y, 0);
    p2scoretext.Position.SetPoint(x + 100, y, 0);
    Messagetext.Position.SetPoint(x, y + 100, 0);

    Forfeittext.Position.SetPoint(x, 35, 0);
    if hudp2text.text = 'CPU' then
        hudp2sprite.Position.SetPoint(width - 120, 35, 0)
    else
        hudp2sprite.Position.SetPoint(width - 160, 35, 0);

    //hud.Position.y:=3.1+(((height/3.1)-(600/3.1))/100);
  //    hudp1.Position.SetPoint();
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
    Shift: TShiftState);
var
    bm: TBitmap;
    i: integer;
begin
    case gamestate of
        gsmenu:
            begin
                if key = VK_RETURN then
                begin
                    play('select');
                    menuarray[menuindex].ModulateColor.Color := clrAqua;
                    menuarray[0].ModulateColor.Color := clrRed;
                    case menuindex of
                        0:
                            begin
                                gamestate := gs1P;
                                turnstate := tsSelect;
                                seqstate := ssBegin;
                                FaderState := fsOut;
                                Fader.Width := 150;
                                Fader.Height := 0;
                                fader.Visible := true;
                                menu.Visible := false;
                                hudp2text.text := 'CPU';
                                hudp2Sprite.Position.SetPoint(width - 120, 35, 0);
                            end;
                        1:
                            begin
                                gamestate := gs2P;
                                turnstate := tsSelect;
                                seqstate := ssBegin;
                                FaderState := fsOut;
                                Fader.Width := 150;
                                Fader.Height := 0;
                                fader.Visible := true;
                                menu.Visible := false;
                                hudp2text.text := 'Player 2';
                                hudp2sprite.Position.SetPoint(width - 160, 35, 0);
                            end;
                        2:
                            begin
                                gamestate := gsOptions;
                                optionsindex := 0;
                                menu.Visible := false;
                                options.visible := true;
                            end;
                        3:
                            close;
                    end;
                end
                else
                begin
                    play('select');
                    menuarray[menuindex].ModulateColor.Color := clrAqua;
                    if key = VK_DOWN then
                    begin
                        menuindex := menuindex + 1;
                        if menuindex > 3 then
                            menuindex := 3;
                    end;

                    if key = VK_UP then
                    begin
                        menuindex := menuindex - 1;
                        if menuindex < 0 then
                            menuindex := 0;
                    end;

                    menuarray[menuindex].ModulateColor.color := clrRed;
                end;

            end;

        gsoptions:
            begin
              play('select');
                if key = VK_RETURN then
                begin
                    optionsarray[optionsindex].ModulateColor.Color := clrAqua;
                    optionsarray[0].ModulateColor.Color := clrRed;
                    case optionsindex of
                        0:
                            begin
                                gamestate := gsMenu;
                                menuindex := 0;
                                options.visible := false;
                                menu.Visible := true;
                            end;
                    end;
                end
                else
                begin
                    optionsarray[optionsindex].ModulateColor.Color := clrAqua;
                    if key = VK_DOWN then
                    begin
                        optionsindex := optionsindex + 1;
                        if optionsindex > 0 then
                            optionsindex := 0;
                    end;

                    if key = VK_UP then
                    begin
                        optionsindex := optionsindex - 1;
                        if optionsindex < 0 then
                            optionsindex := 0;
                    end;

                    optionsarray[optionsindex].ModulateColor.color := clrRed;
                end;
                {  if key=VK_SPACE then
                  begin
                    bm:=tbitmap.create;
                    hud.Visible:=true;
                    for i:=0 to 9 do
                    begin
                      hudp2.roll(36);
                      if hudp2.rollAngle > 360 then
                        hudp2.rollAngle := hudp1.rollAngle - 360;
                      glsceneviewer1.Buffer.Render;
                    glsceneviewer1.Buffer.CreateSnapShotBitmap.SaveToFile('c:\'+inttostr(i)+'.bmp');
                   // bm.savetofile('c:\'+inttostr(tempcount)+'.bmp');
                    end;
                    bm.free;
                    hud.Visible:=false;
                    end;         }

            end;
        gs1P, gs2P:
            begin
                case seqstate of
                    ssNone:
                        if key = VK_ESCAPE then
                        begin
                            seqstate := ssquit;
                            quit.visible := true;
                            play('select');
                        end;
                    ssQUIT:
                        begin
                            if key = VK_ESCAPE then
                            begin
                                gamestate := gsMenu;
                                seqstate := ssNone;
                                quit.visible := false;
                                menu.Visible := true;
                                menuindex := 0;
                                hud.Visible := false;
                                hudp1sprite.FrameIndex := 0;
                                hudp2sprite.FrameIndex := 0;
                                play('select');
                            end
                            else
                            begin
                                seqstate := ssnone;
                                quit.visible := false;
                                play('select');
                            end;
                        end;
                    ssEnd:
                        begin
                            if key = VK_ESCAPE then
                            begin
                                gamestate := gsMenu;
                                seqstate := ssNone;
                                result.visible := false;
                                menu.Visible := true;
                                menuindex := 0;
                                hud.Visible := false;
                                hudp1sprite.FrameIndex := 0;
                                hudp2sprite.FrameIndex := 0;
                                play('select');
                            end;
                        end;
                end;
            end;
    end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    AsyncTimer1.enabled := false;
    AsyncTimer2.enabled := false;
    ValidMoves.free;
    flippers.free;
end;

procedure TForm1.AsyncTimer1Timer(Sender: TObject);
begin
    glsceneviewer1.Invalidate;
end;

procedure TForm1.AsyncTimer2Timer(Sender: TObject);
begin
    caption := copy(caption, 1, pos(':', caption)) + Format(' %.2f fps', [GLSceneViewer1.FramesPerSecond]);
    GLSceneViewer1.ResetPerformanceMonitor;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  asynctimer1.Enabled:=true;
  asynctimer2.Enabled:=true;
end;

end.

